<?php // $Id: send.form.inc,v 1.13 2009/08/26 23:07:09 vauxia Exp $

/**
 * The default submission page for user-initiated send activities.
 */
function send_form($form_state, $profile, $options = array(), $nids = array()) {
  $profile = (object) $profile;
  $profile->nids = ($nids && !is_array($nids)) ? explode(' ', $nids) : $nids;
  $profile->from_element = 'send_user';
  $profile->to_element = 'send_contact';
  $profile->message_element = 'send_message_compose';

  return array(
    '#send_element' => 'send',
    'send' => array(
      '#type' => 'send',
      '#send' => $profile,
    ),
    'buttons' => array(
      'submit' => array( '#type' => 'submit', '#value' => t('Send') ),
    ),
  );
}

// The #after_build functions may have changed the values.  Use those instead.
function send_form_set_values($form, $form_state, $key) {
  $values = array();
  foreach (element_children($form[$key]) as $item) {
    $element = $form[$key][$item];
    if (function_exists($func = $element['#type'] .'_element_set_value')) {
      $element = $func($element, $form_state);
    }
    $values[$item] = $element['#value'];
  }
  return $values;
}

/**
 * A submit handler that is used for any form that contains a send element. 
 * Added by send_element_process().
 */
function send_submit($form, &$form_state) {

  // Override the send profile's default values with POSTed ones.
  foreach ($form_state['values'] as $name => $value) {
    if ($name) $form_state['send']->$name = $value;
  }

  // We only care about elements and values within the send element.
  $key = $form['#send_element'];
  $key = 'send';

  $values = send_form_set_values($form, $form_state, $key);

  $send =& $form_state['send'];

  // Set up the message values.
  $send->message = $values['message'];
  $send->message['sender'] = $values['from'][0];

  if (!isset($form_state['send_deliver']) || $form_state['send_deliver']) {
    // Call the send API with our processed values.
    if ($ret = send($send, $values['to'], $send->nids, $send->message)) {
      flood_register_event("send $send->name");
    }

    // Redirect the user, if there's not already a redirect on this form.
    unset($form_state['rebuild'], $form_state['storage']);
    if (!isset($form_state['redirect'])) {
      if (count($nids) == 1) {
        $form_state['redirect'] = 'node/'. $nids[0];
      }
      else {
        $form_state['redirect'] = '<front>';
      }
    }
  }
}

/**
 * A submit handler that doesn't actually submit the send request.  Useful
 * for AHAH callbacks and buttons that change the form's state or appearance.
 */
function send_submit_refresh($form, &$form_state) {
  $form_state['send_deliver'] = FALSE;
  send_submit($form, $form_state);

  $form_state['rebuild'] = TRUE;
  $form_state['redirect'] = FALSE;
}

/**
 * The #process handler for the main "send" element.
 */
function send_element_process($element, $edit, &$form_state, $form) {
  global $user;
  module_load_include('inc', 'send', 'includes/send.template');

  if (!isset($form_state['send'])) {
    $form_state['send'] = (object) $element['#send'];
  }

  // Make sure that the send item's submit handler is added to the form.
  if (!isset($form_state['submit_handlers'])) {
     $form_state['submit_handlers'] = array();
  }
  if (!in_array('send_submit', $form_state['submit_handlers'])) {
    $form_state['submit_handlers'][] = 'send_submit';
  }

  if (!isset($element['#parents'])) $element['#parents'] = array();
  $element['#parents'] = array();

  $element['#tree'] = TRUE;

  $element['from'] = array(
    '#type' => $form_state['send']->from_element,
  );
  $element['to'] = array(
    '#type' => $form_state['send']->to_element,
    '#required' => TRUE,
  );

  $element['message'] = array(
    '#type' => $form_state['send']->message_element,
  );
  $element['message']['nids'] = array(
    '#type' => 'value',
    '#value' => $form_state['send']->nids,
  );
  return $element;
}

/**
 * An AHAH callback for the send element.
 */
function send_form_js($element = NULL) {
//TODO
module_load_include('form.inc', 'news', 'includes/news');
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  // Step #4.
  $form = form_get_cache($form_build_id, $form_state);
  $key = $form['#send_element'];

  // Preparing for #5.
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form_state['post'] = $form['#post'] = $_POST;
  $form['#programmed'] = $form['#redirect'] = FALSE;

  // Step #5.
  drupal_process_form($form_id, $form, $form_state);

  // Step #6 and #7 and #8.
  $form = drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);

  $send_form = $form[$key];
  $parents = func_get_args();
  foreach ($parents as $parent) {
    if (isset($send_form[$parent])) $send_form = $send_form[$parent];
  }

  unset($send_form['#prefix'], $send_form['#suffix']);

  // Clear out the form_errors array.  We're not here for validation.
  form_set_error(NULL, '', TRUE);
  drupal_get_messages('error', TRUE);

  $output = theme('status_messages') . drupal_render($send_form);

  // Final rendering callback.
  drupal_json(array('status' => TRUE, 'data' => $output));
}

/**
 * A #process handler for a send_contact element.
 */
function send_contact_element_process($element, $edit, &$form_state, $form) {
  $element['#element_validate'] = array('send_contact_element_validate');
  $element['mail'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Email'),
    '#default_value' => $user->mail,
    '#required'      => TRUE,
    '#size'          => 30,
    '#maxsize'       => 100,
  );
  $element['first_name'] = array(
    '#type'          => 'textfield',
    '#title'         => t('First Name'),
    '#default_value' => $sender['first_name'],
    '#size'          => 30,
    '#maxsize'       => 100,
  );
  $element['last_name'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Last Name'),
    '#default_value' => $sender['last_name'],
    '#size'          => 30,
    '#maxsize'       => 100,
  );

  return $element;
}

/**
 * An #after_build handler for a send_contact element.
 * Set the element value in a format that will be recognized by send_contact.
 */
function send_contact_element_set_value($element, &$form_state) {
  $element['#value'] = array(array(
    'mail' => $element['#value']['mail'],
    'name' => $element['#value']['first_name'] .' '. $element['#value']['last_name'],
  ));
  return $element;
}

/**
 * An #element_validate handler for a send_contact element.
 */
function send_contact_element_validate($element, &$form_state) {
  if (!valid_email_address($element['#value']['mail'])) {
    form_set_error($element['#name'], t('Invalid email address'));
  }
}

/**
 * A #process handler for a send_user element.
 */
function send_user_element_process($element, $edit, &$form_state, $form) {
  $element['name'] = array(
    '#type'          => 'textfield',
    '#title'         => t('User name'),
    '#size'          => 15,
    '#maxsize'       => 100,
    '#autocomplete_path' => '',
  );

  // Special handling: "from" elements, you shouldn't be impersonating.
  if ($element['#name'] == 'from') {
    global $user;
    $element['#value']['name'] = $user->name;
    $element['#access'] = FALSE;
  }
  return $element;
}

/**
 * An #after_build handler for a send_user element.
 * Set the element value in a format that will be recognized by send_contact.
 */
function send_user_element_set_value($element, &$form_state) {
  $account = user_load(array('name' => $element['#value']['name']));
  $element['#value'] = array(array(
    'uid' => $account->uid,
    'mail' => $account->mail,
    'name' => strip_tags(theme('username', $account)),
  ));
  return $element;
}

/**
 * An #element_validate handler for a send_user element.
 */
function send_user_element_validate($element, &$form_state) {
}

/**
 * A #process handler for a send_multiple element.
 */
function send_multiple_element_process($element, $edit, &$form_state, $form) {
  $element['name'] = array(
    '#element_validate' => array('send_multiple_element_validate'),
    '#type'          => 'textarea',
    '#title'         => t('Recipients'),
  );
  return $element;
}

/**
 * An #element_validate handler for a send_contact element.
 */
function send_multiple_element_validate($element, &$form_state) {
}

/**
 * A #process handler for a send_message element.
 */
function send_message_compose_element_process($element, $edit, &$form_state, $form) {

  $element['#theme'] = 'send_message_compose_element';
  $element['#after_build'] = array('send_message_compose_element_set_value');

  $send =& $form_state['send'];
  if ($edit) $send->message = $edit;

  $templates = array();
  foreach (send_template_list($send) as $t) {
    $templates[$t->stid] = $t->name;
  }

  if (isset($send->message['admin']['template'])) {
    $template = $send->message['admin']['template'];
  }
  elseif (isset($send->template)) {
    $template = $send->template;
  }
  if (!$template) {
    $template = key($templates);
  }
  $template = send_template_load($template);

  $element['admin'] = array(
    '#prefix' => '<div id="message-admin">',
    '#suffix' => '</div>',
  );

  $element['admin']['template'] = array(
    '#type'          => 'select',
    '#title'         => t('Message template'),
    '#options'       => $templates,
    '#default_value' => $template->stid,
    '#access'        => count($templates) > 1,
    '#parents' => array_merge($element['#parents'], array('template')),
  );

  if (isset($send->message['build_mode'])) {
    $build_mode = $send->message['build_mode'];
  }
  else {
    $build_mode = $template->build_mode ? $template->build_mode : 'teaser';
  }

  $element['admin']['build_mode'] = array(
    '#type' => 'select',
    '#title' => t('Display nodes as'),
    '#options' => send_template_build_modes(),
    '#default_value' => $build_mode,
    '#access' => ($template->build_mode_editable) && $element['nids']['#value'],
    '#parents' => array_merge($element['#parents'], array('build_mode')),
  );

  $element['admin']['template_submit'] = array(
    '#type'          => 'submit',
    '#value'         => t('Change template'),
    '#access'        => ($element['admin']['template']['#access'] || $element['admin']['build_mode']['#access']),
    '#submit'        => array('send_submit_refresh'),
    '#ahah' => array(
      'path' => 'send/js/send_form/'. $element['#parents'][0] .'/content',
      'wrapper' => 'message-content',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );

  // Add an auto-submit action to the template selection.
  if ($element['admin']['template']['#access'] || $element['admin']['build_mode']['#access']) {
    $js = 'Drupal.behaviors.sendMessageForm = function(context) {
      $("#'. $element['#id'] .'-admin-template-submit").hide();
      $("select").change( function() {
        $("#'. $element['#id'] .'-admin-template-submit").mousedown();
        $("#'. $element['#id'] .'-admin-template-submit").click();
      });
    }';
    drupal_add_js($js, 'inline');
  }

  if ($template->subject_editable) {
    if (!$subject = $send->message['subject']) $subject = $template->subject;
    $element['content']['subject_display'] = array(
      '#type' => 'textfield',
      '#title' => t('Subject'),
      '#default_value' => $subject,
      '#parents' => array_merge($element['#parents'], array('subject_display')),
    );
    $element['content']['subject'] = array(
      '#type' => 'value',
      '#value' => $subject,
      '#parents' => array_merge($element['#parents'], array('subject')),
    );
  }
  else {
    $element['content']['subject'] = array(
      '#type' => 'markup',
      '#value' => $template->subject,
      '#prefix' => '<h4>',
      '#suffix' => '</h4>',
    );
  }
  
  // Respect the order that was sent through by table-dragging code.
  if (isset($send->message['content'])) {
    foreach ($send->message['content'] as $key => $val) {
      if (!in_array($key, array('subject', 'subject_display'))) {
        $element['content'][$key] = array();
      }
    }
  }

  $element['content']['message']['content'] = array(
    '#type'  => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $send->message['content']['message']['content'],
  );
  $element['content']['message']['weight'] = array( '#type' => 'hidden' );

  $body = '';

  foreach ($send->nids as $nid) {
    $node = node_load($nid);
    $node->build_mode = $build_mode;
    $teaser = $node->build_mode == 'teaser';
    $content = node_view($node, $teaser);
    $element['content']['node-'. $nid] = array(
      '#type' => 'markup',
      '#value' => $content,
    );
    $element['content']['node-'. $nid]['nid'] = array(
      '#type' => 'hidden',
      '#value' => $nid,
    );

    // TODO make this conditional.
    $element['content']['node-'. $nid]['weight'] = array('#type' => 'hidden');
  }

  $element['content']['#theme'] = 'send_draggable';

  return $element;
}

/**
 * A #process handler for a send_message element.
 */
function send_message_edit_element_process($element, $edit, &$form_state, $form) {
  $send =& $form_state['send'];
  if ($edit) $send->message = $edit;

  if (!$send->message['body']) {
    $value = send_message_compose_element_set_value($element, $form_state);
    $send->message['body'] = $value['#value']['body'];
  }
  $element['subject'] = array(
    '#type' => 'textfield',
    '#default_value' => $send->message['subject'],
  );
  $element['body'] = array(
    '#type' => 'textarea',
    '#default_value' => $send->message['body'],
    '#rows' => 30,
  );
  $element['format'] = filter_form(FILTER_FORMAT_DEFAULT);
  $element['content']['message']['content'] = array(
    '#type' => 'hidden',
    '#value' => $send->message['content']['message']['content'],
  );
  return $element;
}

/**
 * An #after_build handler for a send_message element.
 * Set the element value to something that will be understood by the Send API.
 */
function send_message_compose_element_set_value($element, &$form_state) {
  $send    = $form_state['send'];
  if (!$send->message) return $element;
  $message = $send->message['content']['message']['content'];
  $subject = $send->message['subject'];
  $params  = $send->message;
  $template = send_template_load($send->message['template']);

  $element['#value'] = array_merge($send->message, array(
    'subject' => theme('send_subject', $subject, $profile, $template, $params),
    'body'    => theme('send_body', $message, $profile, $template, $params),
    'content' => $send->message['content'],
    'message' => $message,
  ));
  return $element;
}
